from geomalgo.base1d cimport *
from geomalgo.base2d cimport *
from geomalgo.base3d cimport *
from geomalgo.grid2d cimport *
from geomalgo.inclusion cimport *
from geomalgo.intersection cimport *
from geomalgo.polar cimport *
from geomalgo.triangulation cimport *
